class ProbfitWarning(RuntimeWarning):
    pass


class SmallIntegralWarning(ProbfitWarning):
    pass


class SmallDivisionWarning(ProbfitWarning):
    pass


class SharedExtendeeExtenderParameter(ProbfitWarning):
    pass


class LogWarning(ProbfitWarning):
    pass
